@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2013-2018 Free Software Foundation, Inc.
+   Copyright 2013-2023 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,5 +52,5 @@ int main() {
   D* pd = &d;
   d.m();
 
-  /* stop outside */
+  return 0; /* stop outside */
 }
